#ifndef __TCYLINDER_HH
#define __TCYLINDER_HH
//
// Project : lib3D
// File    : TCylinder.hh
// Purpose : class representing a cylinder
// Author  : Ronald Kriemann
//
// arch-tag: b33e1742-7d22-45f3-8f7b-c02ccb83361e
//

#include "TGraphObject.hh"

namespace RGL
{

//
//
//
class TCylinder : public TGraphObject
{
protected:
    // is the cylinder closed or open
    bool  _closed;
    
public:
    
    //////////////////////////////////////////////
    //
    // constuctor and destructor
    //

    TCylinder ( bool closed = true );

    virtual ~TCylinder ();

    //////////////////////////////////////////////////
    //
    // raytracing methods
    //

    // compute intersection with ray
    // - return true if intersected, else false
    virtual bool intersect ( const TRay & ray, real & dist ) const;

    // compute normal at point
    virtual bool calc_normal ( const TVector & point,
                               TVector       & normal ) const;

    //////////////////////////////////////////////////
    //
    // misc. methods
    //

    // store bounding box defined by xy, xz and yz planes
    // in given vectors
    virtual bool bounding_box ( TVector & min, TVector & max ) const;

    //////////////////////////////////////////////////
    //
    // virtual constructors
    //

    virtual TCylinder * clone () const { return new TCylinder; }
    virtual TCylinder * copy  () const;
};

} // namespace

#endif  // __TCYLINDER_HH
