#ifndef __TINTERSECT_HH
#define __TINTERSECT_HH
//
// Project : lib3D
// File    : TIntersect.hh
// Purpose : baseclass for intersection tests
// Author  : Ronald Kriemann
//
// arch-tag: 42e67437-c284-4312-afdc-e678ac8cba42
//

#include "TGraphObject.hh"
#include "TWorld.hh"
#include "TVector.hh"

namespace RGL
{
    
//
// basic intersection test (test all objects)
//
class TIntersect
{
protected:
    // world of objects
    const TWorld  * _world;

public:
    //////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TIntersect ( const TWorld * w = NULL ) : _world(w) {}

    virtual ~TIntersect () {}
    
    //////////////////////////////////////////////
    //
    // access internal data
    //

    virtual void set_world ( const TWorld * w ) { _world = w; }
    
    //////////////////////////////////////////////
    //
    // intersection test
    //

    // return intersected object and store distance
    // - return NULL of no such object can be found
    // - define viewing-frustrum by front and back
    //   (< 0 means not defined [infinity]) 
    virtual TGraphObject * intersect ( const TRay & ray,
                                       real       & distance,
                                       real         front,
                                       real         back ) const;

protected:

    // check if inside frustrum
    virtual bool inside_view ( real dist, real front, real back ) const;
};

}// namespace

#endif // __TINTERSECT_HH
