#ifndef __TLIGHT_HH
#define __TLIGHT_HH
//
// Project : lib3D
// File    : TLight.hh
// Purpose : baseclass for all lights
// Author  : Ronald Kriemann
//
// arch-tag: f61621f4-691b-4b55-bc68-2c4fd547ff2e
//

#include "TVector.hh"
#include "TColour.hh"
#include "TIntersect.hh"
#include "THPDesc.hh"

namespace RGL
{

//
// set up interface
//
class TLight
{
public:
    //////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TLight () {}

    virtual ~TLight () {}
    
    //////////////////////////////////////////////
    //
    // illumination methods
    //

    // calculate resulting colour at point with
    // normal and store in colour (here the material is used)
    virtual void illuminate ( const THPDesc & hp,
                              const TVector & origin,
                              TColour       & colour ) const
    {
        std::cout << "(TLight) illuminate : must be overloaded" << std::endl;
    }

    // test if light is visible from given point
    // by testing with objects in world
    virtual bool is_visible ( const TVector    & point,
                              const TIntersect & intersect ) const
    {
        std::cout << "(TLight) is_visible : must be overloaded" << std::endl;
        return false;
    }

    //////////////////////////////////////////////
    //
    // virtual constructors
    //

    virtual TLight * clone () const { return new TLight; }
    virtual TLight * copy  () const { return clone(); }

protected:
    //////////////////////////////////////////////
    //
    // compute coefficient for different kind
    // of object-light interaction
    //

    real comp_diffuse ( const TGraphObject * obj,
                        const TVector      & normal,
                        const TVector      & l ) const;
    
    real comp_specular ( const TGraphObject * obj,
                         const TVector      & normal,
                         const TVector      & l,
                         const TVector      & v ) const;
};

} // namespace

#endif  // __TLIGHT_HH
