#ifndef __TMARBLE_HH
#define __TMARBLE_HH
//
// Project : lib3D
// File    : TMarble.hh
// Purpose : provides marble like texture
// Author  : Ronald Kriemann
//
// arch-tag: bbd789e5-ffb8-40ea-a8f9-9c085e7cb834
//

#include "TTexture.hh"
#include "TSolidNoise.hh"

namespace RGL
{

//
// defines marble texture
//
class TMarble : public TTexture
{
protected:
    // colours of different marble structure (fixed)
    TColour      _col1, _col2, _col3;

    // frequency of stripes
    real         _freq;

    // scaling of noise
    real         _scale;

    // depth of turbulence
    uint         _depth;

    // for computing noise
    TSolidNoise  _noise;
    
public:
    //////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TMarble ( real freq,
              real scale = 5.0,
              uint depth = 8,
              uint seed  = 0 );

    virtual ~TMarble () {}
    
    //////////////////////////////////////////////
    //
    // return colour at given point
    //

    virtual void colour ( const TVector & v, TColour & col ) const;
};

}// namespace

#endif  // __TMARBLE_HH
