#ifndef __TPARALLELRT_HH
#define __TPARALLELRT_HH
//
// Project : lib3D
// File    : TParallelRT.hh
// Purpose : class for parallel raytracing
// Author  : Ronald Kriemann
//
// arch-tag: 5b4061fb-5b74-45bd-804f-1d4c0ce9560d
//

#include "TRaytracer.hh"

namespace RGL
{

//
// raytraces world with multiple tasks
//
class TParallelRT : public TRaytracer
{
protected:
    // underlying raytracer
    TRaytracer * _raytrace;
    
    // number of processors to use
    uint         _nprocs;

    // size of tiles
    uint         _tile_size;
    
public:
    ///////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TParallelRT ( TRaytracer * rt, uint nprocs, uint tile_size = 32 );

    virtual ~TParallelRT ();

    ///////////////////////////////////////////////
    //
    // access data of raytracer
    //

    TWorld  * world  () { return _raytrace->world();  }
    
    const TWorld  * world  () const { return _raytrace->world();  }
    const TCamera * camera () const { return _raytrace->camera(); }

    virtual void set_camera ( TCamera * c ) { _raytrace->set_camera( c ); }
    virtual void set_world  ( TWorld  * w ) { _raytrace->set_world( w ); }

    virtual void set_background ( const TColour & c ) { _raytrace->set_background( c ); }

    virtual void set_max_depth     ( uint n ) { _max_depth     = n; }
    virtual void set_min_intensity ( real f ) { _min_intensity = f; }
    
    ///////////////////////////////////////////////
    //
    // rendering
    //

    // render world to printer
    virtual void render_world ( TImage & prn, TProgressBar * bar = NULL ) const;

    // render part of image
    virtual void render_tile ( const TTileDesc & tile,
                               TImage          & img,
                               TProgressBar    * bar = NULL ) const;
    
protected:
    ///////////////////////////////////////////////
    //
    // parallel methods
    //

    // renderes all associated tiles
    void render_tiles ( int                 pipe_desc,
                        TImage            & img,
                        TSLL< TTileDesc > & tile_list ) const;
};

} // namespace

#endif  // __TPARALLELRT_HH
