#ifndef __TPINHOLECAMERA_HH
#define __TPINHOLECAMERA_HH
//
// Project : lib3D
// File    : TPinholeCamera.hh
// Purpose : class representing a pinhole camera
// Author  : Ronald Kriemann
//
// arch-tag: a0209ae7-90cd-4ca6-9f84-51a02a11d658
//

#include "TCamera.hh"

namespace RGL
{

//
// projection is defined by origin and direction of camera
// viewing plane is calculated from this (always look up)
//
class TPinholeCamera : public TCamera
{
protected:
    // distance from the pinhole to the view plane
    real      _vp_dist;
    
    // vector from origin to ulc
    TVector   _start_dir;
    
public:
    //////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TPinholeCamera ();

    virtual ~TPinholeCamera ();

    //////////////////////////////////////////////
    //
    // setup ray corresponding to pixel
    // coordinates (i,j), i,j \in [0,1]
    //

    virtual void set_rays ( TArray< TRay > & rays, real i, real j ) const
    {
        rays.set_size( 1 );
            
        rays[0].new_id();
        rays[0].set_initial();
        
        rays[0].origin().set( 1.0, _origin );
        
        rays[0].direction().set( 1.0, _start_dir ).add( i, _vp_right, j, _vp_down );
        rays[0].direction().normalise();
    }
        
private:

    //
    // update internal data
    //
    
    virtual void update ();
};

} // namespace

#endif  // __TPINHOLECAMERA_HH
