#ifndef __TSOBOL_HH
#define __TSOBOL_HH
//
// Project : misc
// File    : TSobol.hh
// Purpose : class for a Sobol sequence generator
// Author  : Ronald Kriemann
//
// arch-tag: 73c17863-5a07-49e5-8388-3c220ab53a97
//

#include "defines.hh"
#include "TVector.hh"
#include "TArray.hh"

namespace RGL
{

// bit count; assumes sizeof(int) >= 32-bit
#define SOBOL_BIT_COUNT 30

//
// generate numbers defined by Sobol sequences
//
// Implementation stems from the GNU Scientific Library
// (just restricted to 6 dimensions)
//
class TSobol
{
protected:
    // dimension of the Sobol sequence
    uint    _dim;

    // data for the Sobol sequence
    uint    _sequence_count;
    double  _last_denominator_inv;
    int     _last_numerator_vec[6];
    int     _v_direction[SOBOL_BIT_COUNT][6];
    
public:
    ////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TSobol ( uint dim ) { init( dim ); }

    ////////////////////////////////////////////
    //
    // access Sobol sequence
    //

    // return next number in sequence
    void next ( TArray< double > & val, double max = 1.0 );

    // return next number in sequence (special version for dim = 3)
    void next ( TVector & v, double max = 1.0 );
    
    // initialise sobol sequence
    void init ( uint dim );
};

}// namespace

#endif  // __TSOBOL_HH
