#ifndef __TSOLIDNOISE_HH
#define __TSOLIDNOISE_HH
//
// Project : lib3D
// File    : TSolidNoise.hh
// Purpose : class for computing solid noise
// Author  : Ronald Kriemann
//
// arch-tag: 4d9a0556-7659-4266-a88d-15e02bcf807a
//

#include <cmath>

#include "TColour.hh"
#include "TVector.hh"

namespace RGL
{

//
// definition of solid noise follows Perlin
// and "Realistic Raytracing" by Peter Shirley
//
class TSolidNoise
{
protected:
    //
    // defines lattice of gradient vectors
    // for noise interpolation
    //

    TVector  _grid[16];

    //
    // defines permutation of vectors
    //

    int      _phi[16];
    
public:
    //////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TSolidNoise ( uint seed = 0 );

    virtual ~TSolidNoise () {}
    
    //////////////////////////////////////////////
    //
    // supply noise and turbulence
    //

    real noise ( const TVector & v ) const;

    real turbulence ( const TVector & v, uint depth = 0 ) const;

protected:
    //////////////////////////////////////////////
    //
    // misc. helping functions
    //

    // weighting function
    real omega ( real t ) const
    {
        t = std::abs( t );

        if ( t < 1.0 )
        {
            real t3 = t*t*t;
            real t4 = t3*t;
            
            return (-6.0 * t4 * t) + (15.0 * t4) - (10.0 * t3) + 1.0;
        }// if
        else
            return 0.0;
    }

    // return lattice value
    real gamma ( long i, long j, long k, const TVector & v ) const
    {
        int  idx;

        idx = _phi[ std::abs(k)       % 16 ];
        idx = _phi[ std::abs(idx + j) % 16 ];
        idx = _phi[ std::abs(idx + i) % 16 ];

        return dot( _grid[idx], v );
    }
};

}// namespace

#endif  // __TSOLIDNOISE_HH
