#ifndef __TSPOTLIGHT_HH
#define __TSPOTLIGHT_HH
//
// Project : lib3D
// File    : TSpotLight.hh
// Purpose : class for a Spot-light
// Author  : Ronald Kriemann
//
// arch-tag: 46571e93-5d47-473f-abe8-50eba4518f3f
//

#include "TLight.hh"

namespace RGL
{

//
// 
//
class TSpotLight : public TLight
{
protected:
    // position and normal direction of light
    TVector   _pos, _normal;

    // angle defining the light-cone
    real      _max_angle;
    
    // colour of light
    TColour   _colour;

    // last object, we hit
    mutable TGraphObject  * _last_obj;
    
public:
    //////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TSpotLight ( const TVector & pos,
                 const TVector & look_at,
                 real            angle,
                 const TColour & light_col );

    virtual ~TSpotLight () {}
    
    //////////////////////////////////////////////
    //
    // illumination methods
    //

    // calculate resulting colour at point with
    // normal and overwrite given base-colour
    virtual void illuminate ( const THPDesc & hp,
                              const TVector & origin,
                              TColour       & colour ) const;

    // test if light is visible from given point
    virtual bool is_visible ( const TVector    & point,
                              const TIntersect & intersect ) const;

    //////////////////////////////////////////////
    //
    // virtual constructors
    //

    virtual TSpotLight * clone () const;
    virtual TSpotLight * copy  () const { return (TSpotLight*) TLight::copy(); }
};

} // namespace

#endif  // __TSPOTLIGHT_HH
