#ifndef __TSUPERSAMPLINGRT_HH
#define __TSUPERSAMPLINGRT_HH
//
// Project : lib3D
// File    : TSuperSamplingRT.hh
// Purpose : class for raytracing by super sampling
// Author  : Ronald Kriemann
//
// arch-tag: c9f19693-5ac6-4cd1-b905-06baede20634
//

#include "TArray.hh"
#include "TRNG.hh"
#include "TRaytracer.hh"

namespace RGL
{

// different super sampling types
enum { SS_JITTER, SS_POISSON };

//
// uses super sampling to compute colour for each pixel
//
class TSuperSamplingRT : public TRaytracer
{
protected:
    // type of pattern, no of samples per pixel
    uint          _pattern;
    uint          _no_of_samples;

    // RNG for creating samples
    mutable TRNG  _rng;
    
public:
    ///////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TSuperSamplingRT ();

    virtual ~TSuperSamplingRT () {}

    ///////////////////////////////////////////////
    //
    // access local data
    //

    virtual void set_super_sampling ( uint n ) { _no_of_samples = n; }
    
    ///////////////////////////////////////////////
    //
    // rendering
    //

    // render part of image
    virtual void render_tile ( const TTileDesc & tile,
                               TImage          & img,
                               TProgressBar    * bar = NULL ) const;
    
protected:
    ///////////////////////////////////////////////
    //
    // super sampling methods
    //

    // build stochastic distribution for anti-aliasing
    virtual void build_distr ( TArray< real > & x_distr,
                               TArray< real > & y_distr ) const;
};

} // namespace

#endif  // __TSUPERSAMPLINGRT_HH
