//
// Project : librender
// File    : TIntersect.cc
// Purpose : baseclass for intersection tests
// Author  : Ronald Kriemann
//
// arch-tag: a8a80ff1-19d5-4556-ae08-e3d75c2f649a
//

#include <assert.h>

#include "TIntersect.hh"

namespace RGL
{
    
//////////////////////////////////////////////
//
// intersection test
//

//
// return intersected object and store distance
// - return NULL of no such object can be found
//
TGraphObject *
TIntersect::intersect ( const TRay & ray,
                        real       & hit_distance,
                        real         front,
                        real         back ) const
{
    // no world, no objects
    if ( _world == NULL )
        return NULL;
    
    //
    // go over each object in the world and calculate
    // intersection
    //

    TWorld::TObjIterator   iter = _world->objects().first();
    TGraphObject         * min_object = NULL;
    real                   min_distance, distance;

    min_distance = -1;
    
    while ( ! iter.eol() )
    {
        if ( iter()->intersect( ray, distance ) )
        {
            if (inside_view( distance, front, back ) &&
                ((min_object == NULL) || (distance < min_distance)))
            {
                min_object   = iter();
                min_distance = distance;
            }// if
        }// if

        ++iter;
    }// while

    //
    // set hit-distance and return hit-object
    //
    
    hit_distance = min_distance;

    return min_object;
}

//
// check if inside frustrum
//
bool
TIntersect::inside_view ( real distance, real front, real back ) const
{
    if ( distance < 0.0 )
        return false;

    //
    // check against viewing volume of camera
    //
    
    if (( front > 0.0 ) && ( distance < front ))
        return false;

    if (( back > 0.0 ) && ( distance > back ))
        return false;

    return true;
}

}// namespace
