//
// Project : lib3D
// File    : TChecker.cc
// Purpose : provides checker board like texture
// Author  : Ronald Kriemann
//
// arch-tag: 44de52f2-1405-4eff-8985-88c9036c691e
//

#include <cmath>

#include "TChecker.hh"

namespace RGL
{

//////////////////////////////////////////////
//
// constructor and destructor
//

TChecker::TChecker ( const TColour & c1,
                     const TColour & c2,
                     const TVector & freq,
                     bool            smooth )
{
    _col1   = c1;
    _col2   = c2;
    _freq   = freq;
    _smooth = smooth;
}

//////////////////////////////////////////////
//
// return colour at given point
//

void
TChecker::colour ( const TVector & v, TColour & col ) const
{
    if ( _smooth )
    {
        real  tx = (1.0 + std::sin( M_PI * v.x() * _freq.x() )) / 2.0;
        real  ty = (1.0 + std::sin( M_PI * v.y() * _freq.y() )) / 2.0;
        real  tz = (1.0 + std::sin( M_PI * v.z() * _freq.z() )) / 2.0;
        
        col.set( tx, _col1, false ).add( 1.0 - tx, _col2, false );
        col.add( ty, _col1, false ).add( 1.0 - ty, _col2, false );
        col.add( tz, _col1, false ).add( 1.0 - tz, _col2, false );
        col.scale( 1.0 / 3.0 );
    }// if
    else
    {
        long  ix = (long) std::floor(v.x() * _freq.x());
        long  iy = (long) std::floor(v.y() * _freq.y());
        long  iz = (long) std::floor(v.z() * _freq.z());

        if ( (ix + iy + iz) % 2 == 0 ) col = _col1;
        else                           col = _col2;
    }// else
}

}// namespace
