//
// Project : lib3D
// File    : TMarble.cc
// Purpose : provides marble like texture
// Author  : Ronald Kriemann
//
// arch-tag: de09de7a-e56d-4b35-bdce-5abd7fb1688f
//

#include "TMarble.hh"

namespace RGL
{

//////////////////////////////////////////////
//
// constructor and destructor
//

TMarble::TMarble ( real freq, real scale, uint depth, uint seed )
        : _noise( seed )
{
    _col1.set_rgb( 0.8,  0.8,  0.8 );
    _col2.set_rgb( 0.4,  0.2,  0.1 );
    _col3.set_rgb( 0.06, 0.04, 0.02 );

    _freq  = freq;
    _scale = scale;
    _depth = depth;
}

//////////////////////////////////////////////
//
// return colour at given point
//

void
TMarble::colour ( const TVector & v, TColour & col ) const
{
    TVector  w;
    real     t;

    t = _scale * _noise.turbulence( w.set( _freq, v ), _depth );
    t = 2.0 * std::abs( std::sin( _freq * ( w.x() + t ) ) );

    if ( t < 1.0 )
    {
        col.set( t, _col1 ).add( 1.0 - t, _col3 );
    }// if
    else
    {
        t -= 1.0;
        col.set( t, _col1 ).add( 1.0 - t, _col2 );
    }// else
}

}// namespace
