//
// Project : lib3D
// File    : TGraphObject.cc
// Purpose : baseclass for all graphical objects
// Author  : Ronald Kriemann
//
// arch-tag: 25fba2c9-2dbd-4fc2-ab37-c40aa732ab2c
//

#include "TGraphObject.hh"

namespace RGL
{

//
// rotate object
//
void
TGraphObject::rotate ( const TVector & u, real angle )
{
    TMatrix  R;
    real     c = std::cos( angle );
    real     s = std::sin( angle );
    real     xx = u.x() * u.x();
    real     yy = u.y() * u.y();
    real     zz = u.z() * u.z();
    real     emcc = 1.0 - c;

    R(0,0) = xx + c * ( 1.0 - xx );
    R(0,1) = u.x() * u.y() * emcc - u.z() * s;
    R(0,2) = u.z() * u.x() * emcc + u.y() * s;

    R(1,0) = u.x() * u.y() * emcc + u.z() * s;
    R(1,1) = yy + c * ( 1.0 - yy );
    R(1,2) = u.y() * u.z() * emcc - u.x() * s;

    R(2,0) = u.z() * u.x() * emcc - u.y() * s;
    R(2,1) = u.y() * u.z() * emcc + u.x() * s;
    R(2,2) = zz + c * ( 1.0 - zz );

    R.invert();
    _trans.mul_right( R );
}

////////////////////////////////////////////////////////
//
// raytracing methods
//

//
// compute intersection with ray
//
bool
TGraphObject::intersect ( const TRay & ray, real & t ) const
{
    std::cout << "(TGraphObject) intersect : to be done" << std::endl;
    return false;
}

//
// compute normal at point
//
bool
TGraphObject::calc_normal ( const TVector & point,
                            TVector       & normal ) const
{
    std::cout << "(TGraphObject) calc_normal : to be done" << std::endl;
    return false;
}

//////////////////////////////////////////////////
//
// virtual constructors
//

TGraphObject *
TGraphObject::copy  () const
{
    TGraphObject * t = clone();

    t->_trans = _trans;
    t->set_texture( _texture );

    return t;
}
    
////////////////////////////////////////////////////////
//
// misc. methods
//

bool
TGraphObject::bounding_box ( TVector & min, TVector & max ) const
{
    std::cout << "(TGraphObject) bounding_box : to be done" << std::endl;
    return false;
}

//
// print object
//
void
TGraphObject::print () const
{
    std::cout << "TGraphObject" << std::endl;
}

//
// represent object by polygons
//

void
TGraphObject::tesselate ( TSLL< TPolygon * > & polygon ) const
{
    std::cout << "(TGraphObject) tesselate : to be done" << std::endl;
}

#if 0
//////////////////////////////////////////////////
//
// matrix transformation
//

//
// update object and normal transformation matrices
//
void
TGraphObject::update_trans_mat ()
{
    // world-trans = inverse( object-trans )
    _trans_world.invert( _trans_obj );

    // normal-trans = inverse( transpose( world-trans ) )
    // also sufficient: normal-trans = transpose( adjoint( world-trans ) )
    
    _trans_normal(0,0) = (_trans_world(2,2) * _trans_world(1,1) -
                          _trans_world(1,2) * _trans_world(2,1));
    _trans_normal(0,1) = (_trans_world(1,2) * _trans_world(2,0) -
                          _trans_world(1,0) * _trans_world(2,2));
    _trans_normal(0,2) = (_trans_world(1,0) * _trans_world(2,1) -
                          _trans_world(2,0) * _trans_world(1,1));
    
    _trans_normal(1,0) = (_trans_world(0,2) * _trans_world(2,1) -
                          _trans_world(2,2) * _trans_world(0,1));
    _trans_normal(1,1) = (_trans_world(2,2) * _trans_world(0,0) -
                          _trans_world(0,2) * _trans_world(2,0));
    _trans_normal(1,2) = (_trans_world(2,0) * _trans_world(0,1) -
                          _trans_world(0,0) * _trans_world(2,1));
    
    _trans_normal(2,0) = (_trans_world(1,2) * _trans_world(0,1) -
                          _trans_world(0,2) * _trans_world(1,1));
    _trans_normal(2,1) = (_trans_world(1,0) * _trans_world(0,2) -
                          _trans_world(1,2) * _trans_world(0,0));
    _trans_normal(2,2) = (_trans_world(0,0) * _trans_world(1,1) -
                          _trans_world(1,0) * _trans_world(0,1));

    // TMatrix  tmp;

    // _trans_world.transpose( tmp );
    // tmp.invert3( _trans_normal );
}

#endif

} // namespace

